#!/usr/bin/env ruby
#
# $Id: ayftest.rb,v 1.6 2003/04/20 21:47:56 danbri Exp $
# A sample scutter implementation, using the framework 
# defined in RDF4R/scutter.rb
#
# by Dan Brickley <danbri@w3.org>
# $Id: ayftest.rb,v 1.6 2003/04/20 21:47:56 danbri Exp $
#
# TODO: this is more a demo app than a test. write some tests and 
#       turn the rest into a general RDF utility eg /usr/bin/webdata
#
# eg start point:
# http://swordfish.rdfweb.org/discovery/2001/08/codepict/scutterplan.rdf

$LOAD_PATH.unshift '../../lib/' # dev't version, use local copy of library.

# How it works:
# Each 'lambda' is basically a Proc.new creating a code fragment that 
# gets called at various lifecycle points by the Web harvester, passing
# the crawler and current doc. This may or may not be the best way to 
# do things, but seems pretty flexible. Need to think about how this 
# might interact with threading, and find out if Ruby is ever going to get
# genuine threads (useful when tcp/ip stuff blocks, eg dns lookups?).
#
# So we basically create a new 'Scutter', register some code, and set it
# running. 
#
# SYNOPSIS
# ========
# basic idea in 8 lines (todo: move to tiny.rb and library docs):
# 
# require 'RDF4R/scutter'
# bot = RDF4R::SimpleScutter.new 'http://rdfweb.org/2003/02/a-z/az-scutterplan.rdf'
# datasink = lambda do |crawler,page|
#   puts crawler.store_graph(page, crawler.uri,"", 'DBI:Pg:scutter1')
# end
# bot.pagehandlers.push datasink
# bot.errorhandlers.push lambda {|e| puts "error: #{e}" }
# bot.run
#
#
# What it does:
# BE WARNED, this code has SIDE EFFECTS.
# - it will try to write to a PostgreSQL triple store
# - it will make a mess of your stdout
# - it will make network connections to random HTTP sites it finds
# - it (on windows at least) may chew up memory resources
#
# On the other hand, it is kind of cute.
# 
# Why it does it:
#
# To show what a basic RDF harvester might do, eg. to acquire data from the 
# Web, and to show where we might plug in smarter traversal heuristics. Current
# library is hardcoded to simply track all the rdfs:seeAlso crossrefs it finds.
#
# This might be of interest if you want to load some data into a local
# store/cache for more efficient query. See the Cwm pastich at 
# http://www.w3.org/2001/12/rubyrdf/pack/tests/scutter/sqlgraph.rb for 
# possible example client code. A FOAFNaut (see http://www.foafnaut.org/)
# aggregator using this approach is planned (once API to db fixed up).

require 'basicrdf'
require 'RDF4R/scutter'

def go(uri)
  foaf='http://xmlns.com/foaf/0.1/' 
  contact="http://www.w3.org/2000/10/swap/pim/contact#" 
  air= 'http://www.megginson.com/exp/ns/airports#'

  ayf = RDF4R::SimpleScutter.new uri
  pagecount=0

  # a code block to output basic info about each RDF page encountered
  # 
  page_summary = lambda do |crawler,page|  
    puts "RDFDOC: count='#{pagecount}': uri:#{crawler.uri} gave RDF graph #{page} \
	with #{page.size} triples\n" 
    pagecount=pagecount+1
  end

  # a code block to see if some page provides nearestAirport information:  
  #
  #  <contact:nearestAirport><wn:Airport air:icao="EGGD" air:iata="BRS"/>...
  #
  airports = lambda do |crawler,page| 

    #page.reg_xmlns 'http://www.megginson.com/exp/ns/airports#', 'air'
   
    rs = page.ask Statement.new(nil, air+"iata", nil)
    rs.objects.each do |a|
      a.graph=page
      puts "AIRPORT: #{a} got airport code in #{crawler.uri})" if (a.to_s =~ /\S/) 
    end					# the 'if' is fix for parser bug
  end

  # stats to be output at start of each loop      
  #
  loopstats = lambda do |s|
    puts "INIT: s.left.size=#{s.left.size} s.seen.size=#{s.seen.size} current: #{s.uri}"
  end

  error_logger = lambda {|e| puts "ERROR: #{e}" }


  #trying to find src of memory bloat:
  #objstats=nil
  #ayf.pagehandlers.push Proc.new {|c,page| 
  #  objstats=Hash.new(0) # empty and recount each loop
  #  ObjectSpace.each_object{|x| 
  #    objstats[x.class.name]=objstats[x.class.name]+1
  #  }
  #  puts "ObjectStats: #{objstats.inspect}\n"
  #}

  datasink = lambda do |crawler,page|
    puts "Storing data from #{crawler.uri}"
    puts "SQL log: "+crawler.store_graph(page, crawler.uri,"", 'DBI:Pg:scutter1')
    puts 
  end


  # register some handlers:
  ayf.pagehandlers.push page_summary, airports, datasink
  ayf.inithandlers.push loopstats
  ayf.errorhandlers.push error_logger 
  ayf.run  # set crawler running!
end 

################################################################# 

start_uri = 'http://rdfweb.org/people/danbri/rdfweb/danbri-foaf.rdf' 
start_uri = ARGV.shift if ARGV.length > 0

go start_uri
